"""
Parse the open neuro blog entries
"""

from typing import List, Optional, Any, Union
from pathlib import Path
from datetime import datetime, date
import re
from abc import ABC

from pydantic import BaseModel, AnyUrl
from dataclasses import dataclass,  asdict
import frontmatter

from openneuro_wiki.utils import flatten_list

class Element(ABC):
    pattern: re.Pattern

    @classmethod
    def parse(cls, text:str) -> Optional[Any]:
        """
        Parse the text to find the thing!
        """
        return cls.pattern.findall(text)

class Authors(Element):
    pattern = re.compile(r"(?<=Project Author\(s\)).*", re.MULTILINE)

    @classmethod
    def parse(cls, text:str) -> List[str]:
        authors = cls.pattern.findall(text)
        authors = flatten_list([a.split(',') for a in authors])
        authors = [a.strip() for a in authors if a]
        return authors

class Links(Element):
    pattern = re.compile(r"http[s]:\S+")
    """shittiest url regex ever"""

class Post(BaseModel):
    title: str
    authors: List[str]
    project_authors: List[str]
    categories: List[str]
    tags: List[str]
    date: Union[datetime, date]
    content: str
    links: List[AnyUrl]


    @classmethod
    def _fix_frontmatter(self, fm) -> dict:
        # remove jekyll-specific fields
        del fm['layout']
        # rename rarely used fields
        rename_fields = [
            ('author', 'authors')
        ]
        for rename in rename_fields:
            if rename[0] in fm.keys():
                fm[rename[1]] = fm[rename[0]]
                del fm[rename[0]]
        return fm


    @classmethod
    def from_str(cls, md:str) -> 'Post':
        # Parse frontmatter from content
        fm = frontmatter.loads(md)
        metadata = cls._fix_frontmatter(fm.metadata)

        project_authors = Authors.parse(fm.content)

        links = fm.metadata.get('links', [])
        links.extend(Links.parse(fm.content))
        # pdb.set_trace()
        metadata['links'] = links
        metadata['project_authors'] = project_authors
        metadata['content'] = fm.content
        return Post(**metadata)


@dataclass
class ON_Repo:
    """
    Container for paths inside the open neuro repo
    """
    repo_base: Path = Path('../open-neuroscience.github.io')
    en_post_dir: Path = Path('content/en/post')

    def __post_init__(self):
        """
        Make non-base paths relative to base
        """
        self.repo_base = Path(self.repo_base).resolve()

        for k,v in asdict(self).items():
            if k == "repo_base":
                continue
            v = self.repo_base / v
            setattr(self, k, v)

    @property
    def en_posts(self) -> dict[str, Path]:
        paths = sorted(list(self.en_post_dir.glob('**/index.md')))
        return {}






